/******************************
	Picture Book program
	Ver. 0.5
	TAB = 4
	Shigeyuki Seko
	8/23/2000
 ******************************/

#define AppType 0x5069426b      /* 'PiBk' */
#define AppVersion 1

#include <Pilot.h>
#include "book.h"

Int Page;
Int SoundSW;

static struct {
	Int		StartF;
	Int		LastPage;
	Int		LastSound;
	Int		LastData1;
	Int		LastData2;
	Int		LastData3;
	Int		LastData4;
} pass;

void GetPassData(void)
{
	if (pass.StartF != 0x06){
		pass.StartF = 0x06;
		pass.LastPage = 0;
		pass.LastSound = OFF;
		pass.LastData1 = 0;
		pass.LastData2 = 0;
		pass.LastData3 = 0;
		pass.LastData4 = 0;
	}
	Page = pass.LastPage;
	SoundSW = pass.LastSound;
}

void SetPassData(void)
{
	pass.LastPage  = Page;
	pass.LastSound = SoundSW;
}

void dispDbgNumber(short x, short y, long count)
{
        char temp[5];
        char temp2[2];
        RectangleType r;
        r.topLeft.x = x + 5;
        r.topLeft.y = y;
        r.extent.x = 20;
        r.extent.y = 12;
        WinEraseRectangle(&r, 0);
        StrIToA(temp, count);
        temp2[0] = '#';
        temp2[1] = 0x0;
        StrCat(temp, temp2);
        WinDrawChars(temp, StrLen(temp), x, y);
}

static void drawBitmap(Int x, Int y, Int id)
{
	VoidHand h;
	VoidPtr p;

	h = DmGet1Resource('Tbmp', id);
	if(h != NULL){
		p = MemHandleLock(h);
		WinDrawBitmap((BitmapPtr)p, x, y);
		MemHandleUnlock(h);
		DmReleaseResource(h);
	}
}


void SndOutput (long freq,int time,int Laud)
{
	SndCommandType	s;

	s.cmd = sndCmdFreqDurationAmp;
	s.param1 = freq;		/* freq (Hz) */
	s.param2 = time;		/* time (ms) */
	s.param3 = Laud;		/* Laud      */
	if (SoundSW==ON)SndDoCmd(0,&s,true);
}

Int	SoundDecode(unsigned char *data,Int len)
{
	Int	i,j,k;
	Int m,n;
	Int start;
	Int head,level,speed,last;
	long sound;
	unsigned char snd[128];

	/* Erase CR code */

	m=0;
	for (n=0;n<len;n++){
		if ((*(data+n)>0x1f)&&(*(data+n)<0x7f)){
			snd[m]=*(data+n);
			m=m+1;
			if (m>=127) return (ERR);
		}
	}
	len=m;

	/* Check psnd header */

	start=0;
	for (k=0;k<len;k++){
		if ((snd[k]=='p')&&(snd[k+1]=='s')&&(snd[k+2]=='n')&&(snd[k+3]=='d')){
			start = k+4;
		}
	}
	if (start == 0) return(ERR);

	/* Decode sound data */

	i=start;
	start=0;
	do{
		if (snd[i]=='{'){
			if (i>=len-7) return (ERR);
			head =  (((snd[i+1])<<4)&0xf0)+((snd[i+2])&0x0f);
			level = (((snd[i+3])<<4)&0xf0)+((snd[i+4])&0x0f);
			speed = (((snd[i+5])<<4)&0xf0)+((snd[i+6])&0x0f);
			last =  (((snd[i+7])<<4)&0xf0)+((snd[i+8])&0x0f);
			i=i+9;
			if ((i<len)&&(snd[i]=='|')){
				start=i+1;
				break;
			}
			else return (ERR);
		}
	}while (i<len);

	if (start==0) return (ERR);
	for (j=start;j<len;j=j+2){
		if (snd[j]=='}') return (OK);
		if (j<=len-1){
			sound = (long)((144-(((snd[j])<<4)&0xf0)+((snd[j+1])&0x0f))*10);
			if (sound > 10) sound=sound+((level-100)*3);
			if ((sound > 20000)||(sound < 10)) sound=20000;
			SndOutput (sound,(Int)(200-speed),50);
		}
	}
	return (ERR);
}

/********************************
	Start
 ********************************/

static void StartApplication(void)
{
	FormPtr			frm;
	FieldPtr 		opPtr;
	unsigned char	data[150];
	Int				len;

	frm = FrmInitForm(1000);	
	FrmSetActiveForm(frm);
	FrmDrawForm(frm);

	drawBitmap(5, 20, 7000);
//	Page=0;
//	SoundSW=ON;
	StrCopy (data,"8/23/00psnd{PHVDVDQM|Y@WKWHWBVEUIVOVMVGV@UKUBTMTJTHUKUHUCTMTGTASOSMSLSKSK}");
	len=StrLen(data);
	if (SoundDecode(data,len)){
		drawBitmap(5, 20, 7000);
	}
}

static void PageUp(void)
{
	if (Page<MaxPage){
		Page=Page+1;
		drawBitmap(14, 25, 7000+Page);
	}
}

static void PageDown(void)
{
	if (Page>1){
		Page=Page-1;
		drawBitmap(14, 25, 7000+Page);
	}
}

/********************************
	Event Loop
 ********************************/

static Boolean MainFormHandleEvent(EventPtr event)
{
	Boolean		handled = false;
	Err			error;
	FormPtr		frm;

	if(event->eType == ctlSelectEvent){
   		if(event->data.ctlEnter.controlID == 5000){			/* Go Start Page */
			Page=1;
			drawBitmap(14, 25, 7000+Page);
		}
  		else if(event->data.ctlEnter.controlID == 5001){	/* Back */
			PageDown();
		}
 		else if(event->data.ctlEnter.controlID == 5002){	/* Next */
			PageUp();
		}
	}

	else if(event->eType == keyDownEvent){
	   if (event->data.keyDown.chr == pageUpChr){	/* Scroll up key presed */
			SndOutput (400,100,50);
			SndOutput (1000,100,50);
			PageUp();
			handled = true;
		}

		else if (event->data.keyDown.chr == pageDownChr){	/* Scroll down key presed */
			SndOutput (1000,100,50);
			SndOutput (400,100,50);
			PageDown();
			handled = true;
		}
	}

	else if (event->eType == menuEvent){	/* Menu event */
    	switch (event->data.menu.itemID){
		  case 3601:		/* about */
	        frm = FrmInitForm(3000);
            FrmDoDialog(frm);
            FrmDeleteForm(frm);
			break;
		  case 3602:						/* Go Start Page */
			Page=1;
			drawBitmap(14, 25, 7000+Page);
			break;
		  case 3603:						/* Back */
			PageDown();
			break;
		  case 3604:						/* Next */
			PageUp();
			break;
		  case 3605:						/* Sound ON */
			SoundSW=ON;
			break;
		  case 3606:						/* Sound OFF */
			SoundSW=OFF;
			break;
		}
		handled = true;
	}

	else if(event->eType == nilEvent){
		handled = true;
	}
	return handled;
}

/********************************
	Event Loop
 ********************************/

static void EventLoop(void)
{
	EventType	event;
	Word		error;
	short 		err;
	
	do{
		EvtGetEvent(&event, 100); 
		if(!SysHandleEvent(&event))
			if (!MainFormHandleEvent(&event))
				FrmHandleEvent(FrmGetActiveForm(), &event);
		if (MenuHandleEvent((void *)0, &event, &err)) continue;
	} 
	while (event.eType != appStopEvent);
}

/********************************
	Main Routine
 ********************************/

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	Err error;

	if(cmd == sysAppLaunchCmdNormalLaunch){
		PrefGetAppPreferencesV10(AppType, AppVersion, &pass, sizeof(pass));
		GetPassData();
		StartApplication();
		EventLoop();
		SetPassData();
		PrefSetAppPreferencesV10(AppType, AppVersion, &pass, sizeof(pass));
	}
	return(0);
}
